@@ -1,11 +1,30 @@
 #Created 05/04/23  to combine two data frames byrows and columns
 #slight modification 05/06/23 to create column names if missing
-vJoin <- function(x,y, rnames=TRUE,cnames=TRUE) {
+#think about adding a key variable instead of using colnames
+#need to check for all unique x[key.variable,] and y[key.variable,]
+vJoin <- function(x,y, rnames=TRUE,cnames=TRUE,key.name=NULL) {
   n.x <- NCOL(x)
   n.y <- NCOL(y)
   n.obsx <- NROW(x)
   n.obsy <- NROW(y)
-  if(is.null(colnames(x))) colnames(x)<- paste0("x",1:n.x)
+  if(!is.null(key.name)) {
+     all.unique.x <- all.unique.y<- NA
+     if(key.name %in% colnames(x))   all.unique.x <- sum(duplicated(x[,key.name]))
+     if(key.name %in% colnames(y)) all.unique.y <- sum(duplicated(y[,key.name]))
+     if (all.unique.x != 0 ) stop  ("key.name values in x  must be unique")
+     if (all.unique.y != 0  ) stop ("key.name values in y must be unique")
+     rnx <- rownames(x)
+     rny <- rownames(y)
+    #  x.ord <- order(x[,key.name])
+#      x <- x[x.ord,]
+#      y.ord <- order(y[,key.name])
+#      y <- y[y.ord,]
+     rownames(x) <- x[,key.name]
+     rownames(y) <- y[,key.name]
+     x <- cbind(x,rnx)
+     y <- cbind(y,rny)
+  }
+  if(is.null(colnames(x))) colnames(x) <- paste0("x",1:n.x)
   if(is.null(colnames(y))) if(cnames) {colnames(y) <- paste0("y",1:n.y) 
           } else {colnames(y) <- paste0("x",1:n.y)}
   if(is.null(rownames(x))) rownames(x) <- paste0("Sx",1:n.obsx)
